@@ -34,6 +34,10 @@
 #include <time.h>
 #include <signal.h>
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "shared/output.h"
 #include "shared/misc.h"
 #include "shared/optparser.h"
@@ -65,6 +69,10 @@ int main(int argc, char **argv)
 	struct sigaction sigact;
 #endif
 
+#if defined(_WIN32)
+    cl_initialize_crypto();
+#endif
+
     if((opts = optparse(NULL, argc, argv, 1, OPT_CLAMDSCAN, OPT_CLAMSCAN, NULL)) == NULL) {
 	mprintf("!Can't parse command line options\n");
 	return 2;
