@@ -24,9 +24,11 @@
 
 #include <time.h>
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "asn1.h"
-#include "sha1.h"
-#include "md5.h"
 #include "bignum.h"
 #include "matcher-hash.h"
 
@@ -40,12 +42,21 @@
 #define OID_1_2_840_113549_1_1_1 "\x2a\x86\x48\x86\xf7\x0d\x01\x01\x01"
 #define OID_rsaEncryption OID_1_2_840_113549_1_1_1
 
+#define OID_1_2_840_113549_1_1_2 "\x2a\x86\x48\x86\xf7\x0d\x01\x01\x02"
+#define OID_md2WithRSAEncryption OID_1_2_840_113549_1_1_2
+
 #define OID_1_2_840_113549_1_1_4 "\x2a\x86\x48\x86\xf7\x0d\x01\x01\x04"
 #define OID_md5WithRSAEncryption OID_1_2_840_113549_1_1_4
 
 #define OID_1_2_840_113549_1_1_5 "\x2a\x86\x48\x86\xf7\x0d\x01\x01\x05"
 #define OID_sha1WithRSAEncryption OID_1_2_840_113549_1_1_5
 
+#define OID_1_2_840_113549_1_1_11 "\x2a\x86\x48\x86\xf7\x0d\x01\x01\x0b"
+#define OID_sha256WithRSAEncryption OID_1_2_840_113549_1_1_11
+
+#define OID_1_2_840_113549_1_1_13 "\x2a\x86\x48\x86\xf7\x0d\x01\x01\x0d"
+#define OID_sha512WithRSAEncryption OID_1_2_840_113549_1_1_13
+
 #define OID_1_2_840_113549_1_7_1 "\x2a\x86\x48\x86\xf7\x0d\x01\x07\x01"
 #define OID_pkcs7_data OID_1_2_840_113549_1_7_1
 
@@ -97,37 +108,19 @@ struct cli_asn1 {
 };
 
 static int map_sha1(fmap_t *map, const void *data, unsigned int len, uint8_t sha1[SHA1_HASH_SIZE]) {
-    SHA1Context ctx;
     if(!fmap_need_ptr_once(map, data, len)) {
 	cli_dbgmsg("map_sha1: failed to read hash data\n");
 	return 1;
     }
-    SHA1Init(&ctx);
-    while(len) {
-	unsigned int todo = MIN(len, map->pgsz);
-	SHA1Update(&ctx, data, todo);
-	data = (uint8_t *)data + todo;
-	len -= todo;
-    }
-    SHA1Final(&ctx, sha1);
-    return 0;
+    return (cl_sha1(data, len, sha1, NULL) == NULL);
 }
 
 static int map_md5(fmap_t *map, const void *data, unsigned int len, uint8_t *md5) {
-    cli_md5_ctx ctx;
     if(!fmap_need_ptr_once(map, data, len)) {
 	cli_dbgmsg("map_md5: failed to read hash data\n");
 	return 1;
     }
-    cli_md5_init(&ctx);
-    while(len) {
-	unsigned int todo = MIN(len, map->pgsz);
-	cli_md5_update(&ctx, data, len);
-	data = (uint8_t *)data + todo;
-	len -= todo;
-    }
-    cli_md5_final(md5, &ctx);
-    return 0;
+    return (cl_hash_data("md5", data, len, md5, NULL) == NULL);
 }
 
 
@@ -264,8 +257,20 @@ static int asn1_expect_rsa(fmap_t *map, const void **asn1data, unsigned int *asn
 	*hashtype = CLI_SHA1RSA; /* sha1withRSAEncryption 1.2.840.113549.1.1.5 */
     else if(obj.size == lenof(OID_md5WithRSAEncryption) && !memcmp(obj.content, OID_md5WithRSAEncryption, lenof(OID_md5WithRSAEncryption)))
 	*hashtype = CLI_MD5RSA; /* md5withRSAEncryption 1.2.840.113549.1.1.4 */
+    else if(obj.size == lenof(OID_md2WithRSAEncryption) && !memcmp(obj.content, OID_md2WithRSAEncryption, lenof(OID_md2WithRSAEncryption))) {
+	cli_dbgmsg("asn1_expect_rsa: MD2 with RSA (not yet supported)\n");
+	return 1;
+    }
+    else if(obj.size == lenof(OID_sha256WithRSAEncryption) && !memcmp(obj.content, OID_sha256WithRSAEncryption, lenof(OID_sha256WithRSAEncryption))) {
+	cli_dbgmsg("asn1_expect_rsa: SHA256 with RSA (not yet supported)\n");
+	return 1;
+    }
+    else if(obj.size == lenof(OID_sha512WithRSAEncryption) && !memcmp(obj.content, OID_sha512WithRSAEncryption, lenof(OID_sha512WithRSAEncryption))) {
+	cli_dbgmsg("asn1_expect_rsa: SHA512 with RSA (not yet supported)\n");
+	return 1;
+    }
     else {
-	cli_dbgmsg("asn1_expect_rsa: OID mismatch\n");
+	cli_dbgmsg("asn1_expect_rsa: OID mismatch (size %u)\n", obj.size);
 	return 1;
     }
     if((ret = asn1_expect_obj(map, &obj.next, &avail, 0x05, 0, NULL))) /* NULL */
@@ -746,8 +751,8 @@ static int asn1_parse_mscat(fmap_t *map, size_t offset, unsigned int size, crtmg
     const uint8_t *message, *attrs;
     unsigned int dsize, message_size, attrs_size;
     cli_crt_hashtype hashtype;
-    SHA1Context ctx;
     cli_crt *x509;
+    void *ctx;
     int result;
     int isBlacklisted = 0;
 
@@ -1037,10 +1042,13 @@ static int asn1_parse_mscat(fmap_t *map, size_t offset, unsigned int size, crtmg
 	    break;
 	}
 
-	SHA1Init(&ctx);
-	SHA1Update(&ctx, "\x31", 1);
-	SHA1Update(&ctx, attrs + 1, attrs_size - 1);
-	SHA1Final(&ctx, sha1);
+    ctx = cl_hash_init("sha1");
+    if (!(ctx))
+        break;
+
+	cl_update_hash(ctx, "\x31", 1);
+	cl_update_hash(ctx, attrs + 1, attrs_size - 1);
+	cl_finish_hash(ctx, sha1);
 
 	if(!fmap_need_ptr_once(map, asn1.content, asn1.size)) {
 	    cli_dbgmsg("asn1_parse_mscat: failed to read encryptedDigest\n");
@@ -1278,16 +1286,21 @@ static int asn1_parse_mscat(fmap_t *map, size_t offset, unsigned int size, crtmg
 	}
 
 	if(hashtype == CLI_SHA1RSA) {
-	    SHA1Init(&ctx);
-	    SHA1Update(&ctx, "\x31", 1);
-	    SHA1Update(&ctx, attrs + 1, attrs_size - 1);
-	    SHA1Final(&ctx, sha1);
+        ctx = cl_hash_init("sha1");
+        if (!(ctx))
+            break;
+
+        cl_update_hash(ctx, "\x31", 1);
+        cl_update_hash(ctx, attrs + 1, attrs_size - 1);
+        cl_finish_hash(ctx, sha1);
 	} else {
-	    cli_md5_ctx ctx;
-	    cli_md5_init(&ctx);
-	    cli_md5_update(&ctx, "\x31", 1);
-	    cli_md5_update(&ctx, attrs + 1, attrs_size - 1);
-	    cli_md5_final(sha1, &ctx);
+        ctx = cl_hash_init("md5");
+        if (!(ctx))
+            break;
+
+        cl_update_hash(ctx, "\x31", 1);
+        cl_update_hash(ctx, attrs + 1, attrs_size - 1);
+        cl_finish_hash(ctx, sha1);
 	}
 
 	if(!fmap_need_ptr_once(map, asn1.content, asn1.size)) {
