@@ -24,6 +24,7 @@
 #ifndef _WIN32
 #include <sys/time.h>
 #endif
+
 #include "ClamBCModule.h"
 #include "ClamBCDiagnostics.h"
 #include "llvm/Analysis/DebugInfo.h"
@@ -126,15 +127,20 @@ void LLVMInitializePowerPCAsmPrinter();
 #undef PACKAGE_URL
 #include "clamav-config.h"
 #endif
+
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+
+extern "C" {
+#include "libclamav/crypto.h"
+}
+
 #include "dconf.h"
 #include "clamav.h"
 #include "clambc.h"
 #include "bytecode.h"
 #include "bytecode_priv.h"
 #include "type_desc.h"
-extern "C" {
-#include "md5.h"
-}
 
 #define MODULE "libclamav JIT: "
 
@@ -1620,7 +1626,7 @@ public:
 			    Value *Dst = convertOperand(func, inst, inst->u.three[0]);
 			    Dst = Builder.CreatePointerCast(Dst, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Val = convertOperand(func, Type::getInt8Ty(Context), inst->u.three[1]);
-			    Value *Len = convertOperand(func, Type::getInt32Ty(Context), inst->u.three[2]);
+			    Value *Len = convertOperand(func, Type::getInt64Ty(Context), inst->u.three[2]);
 #ifdef LLVM30
 			    CallInst *c = Builder.CreateCall5(CF->FMemset, Dst, Val, Len,
 								ConstantInt::get(Type::getInt32Ty(Context), 1),
@@ -1641,7 +1647,7 @@ public:
 			    Dst = Builder.CreatePointerCast(Dst, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Src = convertOperand(func, inst, inst->u.three[1]);
 			    Src = Builder.CreatePointerCast(Src, PointerType::getUnqual(Type::getInt8Ty(Context)));
-			    Value *Len = convertOperand(func, Type::getInt32Ty(Context), inst->u.three[2]);
+			    Value *Len = convertOperand(func, Type::getInt64Ty(Context), inst->u.three[2]);
 #ifdef LLVM30
 			    CallInst *c = Builder.CreateCall5(CF->FMemcpy, Dst, Src, Len,
 								ConstantInt::get(Type::getInt32Ty(Context), 1),
@@ -1662,7 +1668,7 @@ public:
 			    Dst = Builder.CreatePointerCast(Dst, PointerType::getUnqual(Type::getInt8Ty(Context)));
 			    Value *Src = convertOperand(func, inst, inst->u.three[1]);
 			    Src = Builder.CreatePointerCast(Src, PointerType::getUnqual(Type::getInt8Ty(Context)));
-			    Value *Len = convertOperand(func, Type::getInt32Ty(Context), inst->u.three[2]);
+			    Value *Len = convertOperand(func, Type::getInt64Ty(Context), inst->u.three[2]);
 #ifdef LLVM30
 			    CallInst *c = Builder.CreateCall5(CF->FMemmove, Dst, Src, Len,
 								ConstantInt::get(Type::getInt32Ty(Context), 1),
@@ -1896,7 +1902,7 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
     std::vector<constType*> args;
     args.push_back(PointerType::getUnqual(Type::getInt8Ty(Context)));
     args.push_back(Type::getInt8Ty(Context));
-    args.push_back(Type::getInt32Ty(Context));
+    args.push_back(Type::getInt64Ty(Context));
     args.push_back(Type::getInt32Ty(Context));
 #ifdef LLVM30
     args.push_back(Type::getInt1Ty(Context));
@@ -1905,9 +1911,9 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
 					       args, false);
     CF->FMemset = Function::Create(FuncTy_3, GlobalValue::ExternalLinkage,
 #ifdef LLVM30
-					 "llvm.memset.p0i8.i32",
+					 "llvm.memset.p0i8.i64",
 #else
-					 "llvm.memset.i32",
+					 "llvm.memset.i64",
 #endif
 					 M);
     CF->FMemset->setDoesNotThrow();
@@ -1916,7 +1922,7 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
     args.clear();
     args.push_back(PointerType::getUnqual(Type::getInt8Ty(Context)));
     args.push_back(PointerType::getUnqual(Type::getInt8Ty(Context)));
-    args.push_back(Type::getInt32Ty(Context));
+    args.push_back(Type::getInt64Ty(Context));
     args.push_back(Type::getInt32Ty(Context));
 #ifdef LLVM30
     args.push_back(Type::getInt1Ty(Context));
@@ -1925,9 +1931,9 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
 					       args, false);
     CF->FMemmove = Function::Create(FuncTy_4, GlobalValue::ExternalLinkage,
 #ifdef LLVM30
-					  "llvm.memmove.p0i8.i32",
+					  "llvm.memmove.p0i8.i64",
 #else
-					  "llvm.memmove.i32",
+					  "llvm.memmove.i64",
 #endif
 					  M);
     CF->FMemmove->setDoesNotThrow();
@@ -1935,9 +1941,9 @@ static void addFunctionProtos(struct CommonFunctions *CF, ExecutionEngine *EE, M
 
     CF->FMemcpy = Function::Create(FuncTy_4, GlobalValue::ExternalLinkage,
 #ifdef LLVM30
-					 "llvm.memcpy.p0i8.p0i8.i32",
+					 "llvm.memcpy.p0i8.p0i8.i64",
 #else
-					 "llvm.memcpy.i32",
+					 "llvm.memcpy.i64",
 #endif
 					 M);
     CF->FMemcpy->setDoesNotThrow();
@@ -2201,15 +2207,12 @@ int cli_vm_execute_jit(const struct cli_all_bc *bcs, struct cli_bc_ctx *ctx,
 static unsigned char name_salt[16] = { 16, 38, 97, 12, 8, 4, 72, 196, 217, 144, 33, 124, 18, 11, 17, 253 };
 static void setGuard(unsigned char* guardbuf)
 {
-    cli_md5_ctx ctx;
     char salt[48];
     memcpy(salt, name_salt, 16);
     for(unsigned i = 16; i < 48; i++)
-	salt[i] = cli_rndnum(255);
+        salt[i] = cli_rndnum(255);
 
-    cli_md5_init(&ctx);
-    cli_md5_update(&ctx, salt, 48);
-    cli_md5_final(guardbuf, &ctx);
+    cl_hash_data("md5", salt, 48, guardbuf, NULL);
 }
 
 static void addFPasses(FunctionPassManager &FPM, bool trusted, const TargetData *TD)
