@@ -32,6 +32,10 @@
 #endif
 #include <fcntl.h>
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "cltypes.h"
 #include "others.h"
 #include "mspack.h"
