@@ -32,6 +32,10 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "cltypes.h"
 #include "others.h"
 #include "cpio.h"
@@ -157,7 +161,7 @@ int cli_scancpio_old(cli_ctx *ctx)
 	    if(ret == CL_EMAXFILES) {
 		return ret;
 	    } else if(ret == CL_SUCCESS) {
-		ret = cli_map_scan(*ctx->fmap, pos, filesize, ctx);
+		ret = cli_map_scan(*ctx->fmap, pos, filesize, ctx, CL_TYPE_ANY);
 		if(ret == CL_VIRUS)
 		    return ret;
 	    }
@@ -234,7 +238,7 @@ int cli_scancpio_odc(cli_ctx *ctx)
 	if(ret == CL_EMAXFILES) {
 	    return ret;
 	} else if(ret == CL_SUCCESS) {
-	    ret = cli_map_scan(*ctx->fmap, pos, filesize, ctx);
+	    ret = cli_map_scan(*ctx->fmap, pos, filesize, ctx, CL_TYPE_ANY);
 	    if(ret == CL_VIRUS)
 		return ret;
 	}
@@ -313,7 +317,7 @@ int cli_scancpio_newc(cli_ctx *ctx, int crc)
 	if(ret == CL_EMAXFILES) {
 	    return ret;
 	} else if(ret == CL_SUCCESS) {
-	    ret = cli_map_scan(*ctx->fmap, pos, filesize, ctx);
+	    ret = cli_map_scan(*ctx->fmap, pos, filesize, ctx, CL_TYPE_ANY);
 	    if(ret == CL_VIRUS)
 		return ret;
 	}
