@@ -35,6 +35,10 @@
 #include <time.h>
 #include <errno.h>
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "clamav.h"
 #include "others.h"
 #include "dsig.h"
@@ -42,7 +46,6 @@
 #include "cvd.h"
 #include "readdb.h"
 #include "default.h"
-#include "sha256.h"
 
 #define TAR_BLOCKSIZE 512
 
@@ -194,6 +197,11 @@ static void cli_tgzload_cleanup(int comp, struct cli_dbio *dbio, int fdd)
         free(dbio->buf);
         dbio->buf = NULL;
     }
+
+    if (dbio->hashctx) {
+        cl_hash_destroy(dbio->hashctx);
+        dbio->hashctx = NULL;
+    }
 }
 
 static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, unsigned int options, struct cli_dbio *dbio, struct cli_dbinfo *dbinfo)
@@ -312,7 +320,13 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 	dbio->readsize = dbio->size < dbio->bufsize ? dbio->size : dbio->bufsize - 1;
 	dbio->bufpt = NULL;
 	dbio->readpt = dbio->buf;
-	sha256_init(&dbio->sha256ctx);
+    if (!(dbio->hashctx)) {
+        dbio->hashctx = cl_hash_init("sha256");
+        if (!(dbio->hashctx)) {
+            cli_tgzload_cleanup(compr, dbio, fdd);
+            return CL_EMALFDB;
+        }
+    }
 	dbio->bread = 0;
 
 	/* cli_dbgmsg("cli_tgzload: Loading %s, size: %u\n", name, size); */
@@ -346,7 +360,12 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 			cli_tgzload_cleanup(compr, dbio, fdd);
 			return CL_EMALFDB;
 		    }
-		    sha256_final(&dbio->sha256ctx, hash);
+            cl_finish_hash(dbio->hashctx, hash);
+            dbio->hashctx = cl_hash_init("sha256");
+            if (!(dbio->hashctx)) {
+                cli_tgzload_cleanup(compr, dbio, fdd);
+                return CL_EMALFDB;
+            }
 		    if(memcmp(db->hash, hash, 32)) {
 			cli_errmsg("cli_tgzload: Invalid checksum for file %s\n", name);
 			cli_tgzload_cleanup(compr, dbio, fdd);
@@ -567,6 +586,7 @@ int cl_cvdverify(const char *file)
 	fclose(fs);
 	return CL_EMEM;
     }
+    engine->cb_stats_submit = NULL; /* Don't submit stats if we're just verifying a CVD */
 
     ret = cli_cvdload(fs, engine, NULL, CL_DB_STDOPT | CL_DB_PUA, !!cli_strbcasestr(file, ".cld"), file, 1);
 
@@ -586,6 +606,8 @@ int cli_cvdload(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigne
 	struct cli_dbinfo *dbinfo = NULL;
 	char *dupname;
 
+    dbio.hashctx = NULL;
+
     cli_dbgmsg("in cli_cvdload()\n");
 
     /* verify */
