@@ -26,6 +26,10 @@
 #include <string.h>
 #include <assert.h>
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "others.h"
 
 #include "disasmpriv.h"
