@@ -35,6 +35,10 @@
 
 #include <assert.h>
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "clamav.h"
 #include "others.h"
 #include "htmlnorm.h"
