@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -30,6 +30,10 @@
 #include <unistd.h>
 #endif
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "clamav.h"
 #include "filetypes.h"
 #include "others.h"
@@ -39,10 +43,14 @@
 #include "textdet.h"
 #include "default.h"
 #include "iowrap.h"
+#include "mbr.h"
+#include "gpt.h"
 
 #include "htmlnorm.h"
 #include "entconv.h"
 #include "mpool.h"
+#define UNZIP_PRIVATE
+#include "unzip.h"
 
 static const struct ftmap_s {
     const char *name;
@@ -102,10 +110,16 @@ static const struct ftmap_s {
     { "CL_TYPE_ISO9660",	CL_TYPE_ISO9660		},
     { "CL_TYPE_JAVA",		CL_TYPE_JAVA		},
     { "CL_TYPE_DMG",		CL_TYPE_DMG		},
+    { "CL_TYPE_MBR",        CL_TYPE_MBR     },
+    { "CL_TYPE_GPT",        CL_TYPE_GPT     },
+    { "CL_TYPE_APM",        CL_TYPE_APM     },
     { "CL_TYPE_XAR",		CL_TYPE_XAR		},
     { "CL_TYPE_PART_ANY",	CL_TYPE_PART_ANY	},
     { "CL_TYPE_PART_HFSPLUS",	CL_TYPE_PART_HFSPLUS	},
     { "CL_TYPE_XZ",     	CL_TYPE_XZ      	},
+    { "CL_TYPE_OOXML_WORD",	CL_TYPE_OOXML_WORD     	},
+    { "CL_TYPE_OOXML_PPT",	CL_TYPE_OOXML_PPT     	},
+    { "CL_TYPE_OOXML_XL",	CL_TYPE_OOXML_XL     	},
     { NULL,			CL_TYPE_IGNORED		}
 };
 
@@ -244,7 +258,73 @@ cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine, cli_file_t
 		    cli_dbgmsg("Recognized POSIX tar file\n");
 		    return CL_TYPE_POSIX_TAR;
 	    }
-	}
+	} else if (ret == CL_TYPE_ZIP && bread > 2*(SIZEOF_LH+5)) {
+            const char lhdr_magic[4] = {0x50,0x4b,0x03,0x04};
+            const unsigned char *zbuff = buff;
+            uint32_t zread = bread;
+            uint64_t zoff = bread;
+            const unsigned char * znamep = buff;
+            int32_t zlen = bread;
+            int lhc = 0;
+            int zi;
+            
+            for (zi=0; zi<32; zi++) {
+                znamep = cli_memstr(znamep, zlen, lhdr_magic, 4);
+                if (NULL != znamep) {
+                    znamep += SIZEOF_LH;
+                    zlen = zread - (znamep - zbuff);
+                    if (zlen > 4) { /* Ensure we've mapped for OOXML filename compare */
+                        if (0 == memcmp(znamep, "xl/", 3)) {
+                            cli_dbgmsg("Recognized OOXML XL file\n");
+                            return CL_TYPE_OOXML_XL;
+                        } else if (0 == memcmp(znamep, "ppt/", 4)) {
+                            cli_dbgmsg("Recognized OOXML PPT file\n");
+                            return CL_TYPE_OOXML_PPT;                        
+                        } else if (0 == memcmp(znamep, "word/", 5)) {
+                            cli_dbgmsg("Recognized OOXML Word file\n");
+                            return CL_TYPE_OOXML_WORD;
+                        }
+                        if (++lhc > 2)
+                            break; /* only check first three zip headers */
+                    }
+                    else {
+                        znamep = NULL; /* force to map more */
+                    }
+                }
+
+                if (znamep == NULL) {
+                    if (map->len-zoff > SIZEOF_LH) {
+                        zoff -= SIZEOF_LH+5; /* remap for SIZEOF_LH+filelen for header overlap map boundary */ 
+                        zread = MIN(MAGIC_BUFFER_SIZE, map->len-zoff);
+                        zbuff = fmap_need_off_once(map, zoff, zread);
+                        if (zbuff == NULL) {
+                            cli_dbgmsg("cli_filetype2: error mapping data for OOXML check\n");
+                            return CL_TYPE_ERROR;
+                        }
+                        zoff += zread;
+                        znamep = zbuff;
+                        zlen = zread;
+                    }
+                    else {
+                        break; /* end of data */
+                    }
+                }
+            }
+        } else if (ret == CL_TYPE_MBR) {
+            /* given filetype sig type 0 */
+            int iret = cli_mbr_check(buff, bread, map->len);
+            if (iret == CL_TYPE_GPT) {
+                cli_dbgmsg("Recognized GUID Partition Table file\n");
+                return CL_TYPE_GPT;
+            }
+            else if (iret == CL_CLEAN) {
+                return CL_TYPE_MBR;
+            }
+
+            /* re-detect type */
+            cli_dbgmsg("Recognized binary data\n");
+            ret = CL_TYPE_BINARY_DATA;
+        }
     }
 
     if(ret >= CL_TYPE_TEXT_ASCII && ret <= CL_TYPE_BINARY_DATA) {
