@@ -35,6 +35,10 @@
 #include <stdlib.h>
 #include "clamav.h"
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "cltypes.h"
 #include "others.h"
 #include "ole2_extract.h"
@@ -511,9 +515,10 @@ ole2_walk_property_tree(ole2_header_t * hdr, const char *dir, int32_t prop_index
     int32_t         idx, current_block, i, curindex;
     char           *dirname;
     ole2_list_t     node_list;
-    int             ret;
+    int             ret, func_ret;
 
     ole2_listmsg("ole2_walk_property_tree() called\n");
+    func_ret = CL_SUCCESS;
     ole2_list_init(&node_list);
 
     ole2_listmsg("rec_level: %d\n", rec_level);
@@ -600,10 +605,17 @@ ole2_walk_property_tree(ole2_header_t * hdr, const char *dir, int32_t prop_index
                 continue;
             }
             hdr->sbat_root_start = prop_block[idx].start_block;
-            if ((prop_block[idx].child != -1) &&
-                    (ret = ole2_walk_property_tree(hdr, dir, prop_block[idx].child, handler, rec_level + 1, file_count, ctx, scansize)) != CL_SUCCESS) {
-                ole2_list_delete(&node_list);
-                return ret;
+            if (prop_block[idx].child != -1) {
+                ret = ole2_walk_property_tree(hdr, dir, prop_block[idx].child, handler, rec_level + 1, file_count, ctx, scansize);
+                if (ret != CL_SUCCESS) {
+                    if ((ctx->options & CL_SCAN_ALLMATCHES) && (ret == CL_VIRUS)) {
+                        func_ret = ret;
+                    }
+                    else {
+                        ole2_list_delete(&node_list);
+                        return ret;
+                    }
+                }
             }
             if (prop_block[idx].prev != -1) {
 	        if ((ret=ole2_list_push(&node_list, prop_block[idx].prev)) != CL_SUCCESS) {
@@ -623,24 +635,37 @@ ole2_walk_property_tree(ole2_header_t * hdr, const char *dir, int32_t prop_index
             if (ctx && ctx->engine->maxfiles && ctx->scannedfiles + *file_count > ctx->engine->maxfiles) {
                 cli_dbgmsg("OLE2: files limit reached (max: %u)\n", ctx->engine->maxfiles);
                 ole2_list_delete(&node_list);
-                return CL_BREAK;
+                return CL_EMAXFILES;
             }
             if (!ctx || !(ctx->engine->maxfilesize) || prop_block[idx].size <= ctx->engine->maxfilesize || prop_block[idx].size <= *scansize) {
                 (*file_count)++;
                 *scansize -= prop_block[idx].size;
                 ole2_listmsg("running file handler\n");
-                if ((ret = handler(hdr, &prop_block[idx], dir, ctx)) != CL_SUCCESS) {
-                    ole2_listmsg("file handler returned %d\n", ret);
-		    ole2_list_delete(&node_list);
-                    return ret;
+                ret = handler(hdr, &prop_block[idx], dir, ctx);
+                if (ret != CL_SUCCESS) {
+                    if ((ctx->options & CL_SCAN_ALLMATCHES) && (ret == CL_VIRUS)) {
+                        func_ret = ret;
+                    }
+                    else {
+                        ole2_listmsg("file handler returned %d\n", ret);
+                        ole2_list_delete(&node_list);
+                        return ret;
+                    }
                 }
             } else {
                 cli_dbgmsg("OLE2: filesize exceeded\n");
             }
-            if ((prop_block[idx].child != -1) &&
-                    (ret = ole2_walk_property_tree(hdr, dir, prop_block[idx].child, handler, rec_level, file_count, ctx, scansize)) != CL_SUCCESS) {
-                ole2_list_delete(&node_list);
-                return ret;
+            if (prop_block[idx].child != -1) {
+                ret = ole2_walk_property_tree(hdr, dir, prop_block[idx].child, handler, rec_level, file_count, ctx, scansize);
+                if (ret != CL_SUCCESS) {
+                    if ((ctx->options & CL_SCAN_ALLMATCHES) && (ret == CL_VIRUS)) {
+                        func_ret = ret;
+                    }
+                    else {
+                        ole2_list_delete(&node_list);
+                        return ret;
+                    }
+                }
             }
             if (prop_block[idx].prev != -1) {
 	        if ((ret=ole2_list_push(&node_list, prop_block[idx].prev)) != CL_SUCCESS) {
@@ -674,10 +699,17 @@ ole2_walk_property_tree(ole2_header_t * hdr, const char *dir, int32_t prop_index
                 cli_dbgmsg("OLE2 dir entry: %s\n", dirname);
             } else
                 dirname = NULL;
-            if ((prop_block[idx].child != -1) &&
-                    (ret = ole2_walk_property_tree(hdr, dirname, prop_block[idx].child, handler, rec_level + 1, file_count, ctx, scansize)) != CL_SUCCESS) {
-                ole2_list_delete(&node_list);
-                return ret;
+            if (prop_block[idx].child != -1) {
+                ret = ole2_walk_property_tree(hdr, dirname, prop_block[idx].child, handler, rec_level + 1, file_count, ctx, scansize);
+                if (ret != CL_SUCCESS) {
+                    if ((ctx->options & CL_SCAN_ALLMATCHES) && (ret == CL_VIRUS)) {
+                        func_ret = ret;
+                    }
+                    else {
+                        ole2_list_delete(&node_list);
+                        return ret;
+                    }
+                }
             }
             if (prop_block[idx].prev != -1) {
 	        if ((ret=ole2_list_push(&node_list, prop_block[idx].prev)) != CL_SUCCESS) {
@@ -701,7 +733,7 @@ ole2_walk_property_tree(ole2_header_t * hdr, const char *dir, int32_t prop_index
         ole2_listmsg("loop ended: %d %d\n", ole2_list_size(&node_list), ole2_list_is_empty(&node_list));
     }
     ole2_list_delete(&node_list);
-    return CL_SUCCESS;
+    return func_ret;
 }
 
 /* Write file Handler - write the contents of the entry to a file */
@@ -945,6 +977,13 @@ handler_otf(ole2_header_t * hdr, property_t * prop, const char *dir, cli_ctx * c
     }
 
     if (lseek(ofd, 0, SEEK_SET) == -1) {
+        close(ofd);
+        if (ctx && !(ctx->engine->keeptmp))
+            cli_unlink(tempfile);
+
+        free(tempfile);
+        free(buff);
+        cli_bitset_free(blk_bitset);
         return CL_ESEEK;
     }
     ret = cli_magic_scandesc(ofd, ctx);
