@@ -31,6 +31,10 @@
 #include <unistd.h>
 #endif
 
+#if HAVE_PTHREAD_H
+#include <pthread.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include "cltypes.h"
@@ -55,7 +59,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 76
+#define CL_FLEVEL 77
 #define CL_FLEVEL_DCONF	CL_FLEVEL
 #define CL_FLEVEL_SIGTOOL CL_FLEVEL
 
@@ -138,6 +142,35 @@ typedef struct cli_ctx_tag {
 #endif
 } cli_ctx;
 
+#define STATS_ANON_UUID "5b585e8f-3be5-11e3-bf0b-18037319526c"
+#define STATS_MAX_SAMPLES 50
+#define STATS_MAX_MEM 1024*1024
+
+typedef struct cli_flagged_sample {
+    char **virus_name;
+    char md5[16];
+    size_t size; /* A size of zero means size is unavailable (why would this ever happen?) */
+    uint32_t hits;
+    stats_section_t *sections;
+
+    struct cli_flagged_sample *prev;
+    struct cli_flagged_sample *next;
+} cli_flagged_sample_t;
+
+typedef struct cli_clamav_intel {
+    char *hostid;
+    char *host_info;
+    cli_flagged_sample_t *samples;
+    uint32_t nsamples;
+    uint32_t maxsamples;
+    uint32_t maxmem;
+    uint32_t timeout;
+    time_t nextupdate;
+    struct cl_engine *engine;
+#ifdef CL_THREAD_SAFE
+    pthread_mutex_t mutex;
+#endif
+} cli_intel_t;
 
 typedef struct {uint64_t v[2][4];} icon_groupset;
 
@@ -193,6 +226,7 @@ struct cl_engine {
     uint32_t ac_maxdepth;
     char *tmpdir;
     uint32_t keeptmp;
+    uint64_t engine_options;
 
     /* Limits */
     uint64_t maxscansize;  /* during the scanning of archives this size
@@ -284,7 +318,22 @@ struct cl_engine {
     uint64_t maxscriptnormalize; /* max size to normalize scripts */
     uint64_t maxziptypercg; /* max size to re-do zip filetype */
 
-    uint32_t forcetodisk; /* cause memory or map scans to dump to disk first */
+    /* Statistics/intelligence gathering */
+    void *stats_data;
+    clcb_stats_add_sample cb_stats_add_sample;
+    clcb_stats_remove_sample cb_stats_remove_sample;
+    clcb_stats_decrement_count cb_stats_decrement_count;
+    clcb_stats_submit cb_stats_submit;
+    clcb_stats_flush cb_stats_flush;
+    clcb_stats_get_num cb_stats_get_num;
+    clcb_stats_get_size cb_stats_get_size;
+    clcb_stats_get_hostid cb_stats_get_hostid;
+
+    /* Raw disk image max settings */
+    uint32_t maxpartitions;
+
+    /* Engine max settings */
+    uint32_t maxiconspe; /* max number of icons to scan for PE */
 };
 
 struct cl_settings {
@@ -307,6 +356,7 @@ struct cl_settings {
     uint32_t bytecode_timeout;
     enum bytecode_mode bytecode_mode;
     char *pua_cats;
+    uint64_t engine_options;
 
     /* callbacks */
     clcb_pre_cache cb_pre_cache;
@@ -325,7 +375,22 @@ struct cl_settings {
     uint64_t maxscriptnormalize; /* max size to normalize scripts */
     uint64_t maxziptypercg; /* max size to re-do zip filetype */
 
-    uint32_t forcetodisk; /* cause memory or map scans to dump to disk first */
+    /* Statistics/intelligence gathering */
+    void *stats_data;
+    clcb_stats_add_sample cb_stats_add_sample;
+    clcb_stats_remove_sample cb_stats_remove_sample;
+    clcb_stats_decrement_count cb_stats_decrement_count;
+    clcb_stats_submit cb_stats_submit;
+    clcb_stats_flush cb_stats_flush;
+    clcb_stats_get_num cb_stats_get_num;
+    clcb_stats_get_size cb_stats_get_size;
+    clcb_stats_get_hostid cb_stats_get_hostid;
+
+    /* Raw disk image max settings */
+    uint32_t maxpartitions; /* max number of partitions to scan in a disk image */
+
+    /* Engine max settings */
+    uint32_t maxiconspe; /* max number of icons to scan for PE */
 };
 
 extern int (*cli_unrar_open)(int fd, const char *dirname, unrar_state_t *state);
@@ -556,7 +621,7 @@ char *cli_gentemp(const char *dir);
 int cli_gentempfd(const char *dir, char **name, int *fd);
 unsigned int cli_rndnum(unsigned int max);
 int cli_filecopy(const char *src, const char *dest);
-int cli_mapscan(fmap_t *map, off_t offset, size_t size, cli_ctx *ctx);
+int cli_mapscan(fmap_t *map, off_t offset, size_t size, cli_ctx *ctx, cli_file_t type);
 bitset_t *cli_bitset_init(void);
 void cli_bitset_free(bitset_t *bs);
 int cli_bitset_set(bitset_t *bs, unsigned long bit_offset);
