@@ -52,14 +52,16 @@
 #include <malloc.h>
 #endif
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "clamav.h"
 #include "others.h"
-#include "md5.h"
 #include "cltypes.h"
 #include "regex/regex.h"
 #include "ltdl.h"
 #include "matcher-ac.h"
-#include "md5.h"
 
 static unsigned char name_salt[16] = { 16, 38, 97, 12, 8, 4, 72, 196, 217, 144, 33, 124, 18, 11, 17, 253 };
 
@@ -823,13 +825,9 @@ static char *cli_md5buff(const unsigned char *buffer, unsigned int len, unsigned
 {
 	unsigned char digest[16];
 	char *md5str, *pt;
-	cli_md5_ctx ctx;
 	int i;
 
-
-    cli_md5_init(&ctx);
-    cli_md5_update(&ctx, buffer, len);
-    cli_md5_final(digest, &ctx);
+    cl_hash_data("md5", buffer, len, digest, NULL);
 
     if(dig)
 	memcpy(dig, digest, 16);
@@ -903,7 +901,6 @@ char *cli_gentemp(const char *dir)
 
 int cli_gentempfd(const char *dir, char **name, int *fd)
 {
-
     *name = cli_gentemp(dir);
     if(!*name)
 	return CL_EMEM;
@@ -913,10 +910,11 @@ int cli_gentempfd(const char *dir, char **name, int *fd)
      * EEXIST is almost impossible to occur, so we just treat it as other
      * errors
      */
-   if(*fd == -1) {
-	cli_errmsg("cli_gentempfd: Can't create temporary file %s: %s\n", *name, strerror(errno));
-	free(*name);
-	return CL_ECREAT;
+    if(*fd == -1) {
+        cli_errmsg("cli_gentempfd: Can't create temporary file %s: %s\n", *name, strerror(errno));
+        free(*name);
+        *name = NULL;
+        return CL_ECREAT;
     }
 
     return CL_SUCCESS;
