@@ -55,6 +55,10 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "cltypes.h"
 #include "pe.h"
 #include "rebuildpe.h"
