@@ -36,6 +36,10 @@
 #include <clamav.h>
 #include <ctype.h>
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "clamav.h"
 #include "cltypes.h"
 #include "others.h"
