@@ -25,10 +25,10 @@
 
 #include "others.h"
 #include "cltypes.h"
-#include "hashtab.h"
+#include "uniq.h"
 
 typedef struct vba_project_tag {
-	uint32_t *name;
+	char **name;
 	uint32_t *colls;
 	uint32_t *offset;
 	uint32_t *length;	/* for Word 6 macros */
@@ -42,7 +42,7 @@ vba_project_t	*cli_vba_readdir(const char *dir, struct uniq *U, uint32_t which);
 vba_project_t	*cli_wm_readdir(int fd);
 unsigned char	*cli_vba_inflate(int fd, off_t offset, int *size);
 int	cli_scan_ole10(int fd, cli_ctx *ctx);
-char	*cli_ppt_vba_read(int fd);
+char	*cli_ppt_vba_read(int fd, cli_ctx *ctx);
 unsigned char	*cli_wm_decrypt_macro(int fd, off_t offset, uint32_t len,
 					unsigned char key);
 
