@@ -30,6 +30,11 @@
 #if HAVE_CONFIG_H
 #include "clamav-config.h"
 #endif
+
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "cltypes.h"
 #include "pe.h"
 #include "others.h"
