@@ -50,6 +50,10 @@
 #include <syslog.h>
 #endif
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "output.h"
 #include "libclamav/others.h"
 #include "libclamav/str.h"
