@@ -24,6 +24,10 @@
 #include <fcntl.h>
 #include <io.h>
 
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include "libclamav/crypto.h"
+
 #include "others.h"
 #include "shared/misc.h"
 
